//+------------------------------------------------------------------+
//|                                                   Controller.mqh |
//|                                Copyright 2021, Andrei Novichkov. |
//|                                              https://fxstill.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2021, Andrei Novichkov."
#property link      "https://fxstill.com"
#property version   "1.00"

#define NAME (string)"WPRMVC"

#include "..\Model\Model.mqh"
#include "Input.mqh"
#include "..\View\View.mqh"
#include "ButtonObj.mqh"


#define DELOBJ_IF(x) if (CheckPointer(x) != POINTER_INVALID) {delete x; x = NULL;}

class CController {
 public:
                 CController();
                ~CController();
   
           int   Initialize();
           bool  Update();
           int Tick(const int rates_total,
                    const int prev_calculated,
                    const datetime &time[],
                    const double &open[],
                    const double &high[],
                    const double &low[],
                    const double &close[],
                    const long &tick_volume[],
                    const long &volume[],
                    const int &spread[]);
                    
            void ChartEvent(const int id,
                    const long &lparam,
                    const double &dparam,
                    const string &sparam);                    


 protected:
 private:
   CInputManager*  pInput;  
   CModel*         pModel;
   CView*          pView;
   CButtonObj*     pBtn;
};
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CController::CController() {
   pInput = new CInputManager();
   pModel = new CModel();
   pView  = new CView();
   pBtn   = new CButtonObj("BTN");
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CController::~CController() {
   DELOBJ_IF(pInput);
   DELOBJ_IF(pModel);
   DELOBJ_IF(pView);
   DELOBJ_IF(pBtn);
}
//+------------------------------------------------------------------+
int CController::Tick(const int rates_total,const int prev_calculated,const datetime &time[],const double &open[],const double &high[],const double &low[],const double &close[],const long &tick_volume[],const long &volume[],const int &spread[]) {

   return pModel.Tick(rates_total,prev_calculated,time,open,high,low,close,tick_volume,volume,spread);
}

void CController::ChartEvent(const int id,const long &lparam,const double &dparam,const string &sparam) {

      switch (id) {
         case CHARTEVENT_OBJECT_CLICK:
            if (StringCompare(sparam, pBtn.GetName()) == 0) {
               if (pView.GetViewType() == LINE)
                  pView.SetNewViewType(HISTO);
               else pView.SetNewViewType(LINE);   
            }
            break;      
        default:
            break;    
      }//switch (id)
}

int CController::Initialize() {

   if (CheckPointer(pInput) == POINTER_INVALID ||
       CheckPointer(pModel) == POINTER_INVALID ||
       CheckPointer(pView)  == POINTER_INVALID) return INIT_FAILED;
            
   int iResult =  pInput.Initialize();
   if (iResult != INIT_SUCCEEDED) return iResult;
   
   iResult = pView.Initialize(GetPointer(pInput) );
   if (iResult != INIT_SUCCEEDED) return iResult;
   
   iResult =  pModel.Initialize(GetPointer(pInput), GetPointer(pView) );
   if (iResult != INIT_SUCCEEDED) return iResult;
   
   pBtn.Initialize();
   
   return INIT_SUCCEEDED;
} 

bool CController::Update() {
   
   bool bResult = pInput.Update();
   
   return bResult;
}